unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, Menus;

type

  { TfmMain }

  TPlayRec = record
    UserName: string[30];
    ATime: TDateTime;
    Level: Integer;
    Correct: Integer;
    NotCorrect: Integer;
    Mark: Integer;
  end;

  TfmMain = class(TForm)
    imLight: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    laChooseNumber: TLabel;
    laStart: TLabel;
    laLevelSelect: TLabel;
    laLevel: TLabel;
    fmMain: TPanel;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    paAns2: TPanel;
    paAns3: TPanel;
    paAns4: TPanel;
    paAns1: TPanel;
    paAns5: TPanel;
    paAns6: TPanel;
    paCorrect: TPanel;
    paMark: TPanel;
    paUserName: TPanel;
    paFalse: TPanel;
    paTime: TPanel;
    paQuestion: TPanel;
    paState: TPanel;
    tiFinish: TTimer;
    tiTime: TTimer;
    tiQuestion: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure laLevelSelectClick(Sender: TObject);
    procedure laStartClick(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure MenuItem4Click(Sender: TObject);
    procedure MenuItem5Click(Sender: TObject);
    procedure paAns1Click(Sender: TObject);
    procedure tiFinishStartTimer(Sender: TObject);
    procedure tiFinishTimer(Sender: TObject);
    procedure tiQuestionStartTimer(Sender: TObject);
    procedure tiQuestionTimer(Sender: TObject);
    procedure tiTimeStartTimer(Sender: TObject);
    procedure tiTimeTimer(Sender: TObject);
  private
    UserName: string;
    function FindEmptyCell: Integer;
    { private declarations }
  public
    NumbersComp: array [0 .. 999] of TImage;
    Correct: array [1 .. 6] of Boolean;
    CorrectCount, NotCorrectCount: Integer;
    StartTime: TDateTime;
    Remaining: Integer;
    UnderSNum: Integer;

    GLevel: Integer;
    { public declarations }
    function InvertNum(Num: string): string;
    function InvertNumbers(Numbers: string): string;
    procedure ViewNumbers(Numbers: string; var APanel: TPanel;
      Transparent: Boolean = False);
    procedure RemoveAllImagesInPanel(APanel: TPanel);
    function GetNumEnd(Numbers: string): Integer;
    procedure DoGameOne;
    procedure DoUnderSchoolGame;
    procedure FinishGame;
    procedure SaveGame;
    function GetMark: Integer;
  end; 

var
  fmMain: TfmMain;

implementation

uses numberBox, NumbersLib, NumbersGame, Level, About, History;

{ TfmMain }

procedure TfmMain.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  Randomize;
  for i:= 0 to High(NumbersComp) do
    NumbersComp[i]:= nil;
end;

procedure TfmMain.FormKeyPress(Sender: TObject; var Key: char);
begin
  if tiTime.Enabled then
  if (Key in ['1' .. '9']) and (UnderSNum <> 0) then
  begin
    paState.Visible:= True;

    if StrToInt(Key) = UnderSNum then
    begin
      UnderSNum:= 0;
      Inc(CorrectCount);
      ViewNumbers(IntToStr(CorrectCount), paCorrect);
      paState.Caption:= 'صحيح';
      imLight.Show;
      paState.Color:= clGreen;
    end
    else
    begin
      UnderSNum:= 0;
      Inc(NotCorrectCount);
      ViewNumbers(IntToStr(NotCorrectCount), paFalse);
      paState.Caption:= 'خطأ';
      imLight.Hide;
      paState.Color:= clRed;
    end;

    tiQuestion.Enabled:= True;

  end;
end;

procedure TfmMain.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
end;


procedure TfmMain.laLevelSelectClick(Sender: TObject);
begin
  if fmLevel.ShowModal = mrOK then
  begin
    GLevel:= fmLevel.rgLevel.ItemIndex;
    laStart.Enabled:= True;
    case GLevel of
      0: laLevel.Caption:= 'ماقبل المدرسة';
      1: laLevel.Caption:= 'المستوى الأول';
      2: laLevel.Caption:= 'المستوى الثاني';
      3: laLevel.Caption:= 'المستوى الثالث';
      4: laLevel.Caption:= 'المستوى الرابع';
      5: laLevel.Caption:= 'المستوى الخامس';
      6: laLevel.Caption:= 'المستوى السادس';
    end;

    if GLevel = 0 then
    begin
      paQuestion.Height:= 110;
    end
    else
    begin
      paQuestion.Height:= 50;
    end;

    laChooseNumber.Visible:= GLevel = 0;

    paAns1.Visible:=  GLevel <> 0;
    paAns2.Visible:=  GLevel <> 0;
    paAns3.Visible:=  GLevel <> 0;
    paAns4.Visible:=  GLevel <> 0;

    paAns5.Visible:= GLevel > 4;
    paAns6.Visible:= GLevel > 5;
  end;
end;

procedure TfmMain.laStartClick(Sender: TObject);
begin
  if InputQuery('إسم الاعب', 'الرجاء إدخال إسمك', UserName) then
  begin
    paUserName.Caption:= UserName;
    tiFinish.Enabled:= False;
    laLevelSelect.Enabled:= False;
    paMark.Visible:= False;
    CorrectCount:= 0;
    NotCorrectCount:= 0;
    paCorrect.Visible:= True;
    paFalse.Visible:= True;
    ViewNumbers('0', paCorrect, False);
    ViewNumbers('0', paFalse, False);
    Remaining:= 300;
    if GLevel = 0 then
      DoUnderSchoolGame
    else
      DoGameOne;
    StartTime:= Now;
    tiTime.Enabled:= True;
    paTime.Visible:= True;
    paUserName.Visible:= True;
  end;
end;

procedure TfmMain.MenuItem2Click(Sender: TObject);
begin
end;

procedure TfmMain.MenuItem3Click(Sender: TObject);
begin
  Close;
end;

procedure TfmMain.MenuItem4Click(Sender: TObject);
begin
  fmAbout.ShowModal;
end;

procedure TfmMain.MenuItem5Click(Sender: TObject);
begin
  fmHistory.Show;
  fmHistory.ViewHistory;
end;

procedure TfmMain.paAns1Click(Sender: TObject);
var
  Index: Integer;
begin
  Index:= (Sender as TComponent).Tag;
  paState.Visible:= True;
  if Index in [1 .. 6] then
  if Correct[Index] then
  begin
    Inc(CorrectCount);
    ViewNumbers(IntToStr(CorrectCount), paCorrect);
    paState.Caption:= 'صحيح';
    imLight.Show;
    paState.Color:= clGreen;
  end
  else
  begin
    Inc(NotCorrectCount);
    ViewNumbers(IntToStr(NotCorrectCount), paFalse);
    paState.Caption:= 'خطأ';
    imLight.Hide;
    paState.Color:= clRed;
  end;
  paAns1.Enabled:= False;
  paAns2.Enabled:= False;
  paAns3.Enabled:= False;
  paAns4.Enabled:= False;
  paAns5.Enabled:= False;
  paAns6.Enabled:= False;
  tiQuestion.Enabled:= True;
end;

procedure TfmMain.tiFinishStartTimer(Sender: TObject);
begin
end;

procedure TfmMain.tiFinishTimer(Sender: TObject);
begin
  paMark.Visible:= not paMark.Visible;
end;

procedure TfmMain.tiQuestionStartTimer(Sender: TObject);
begin

end;

procedure TfmMain.tiQuestionTimer(Sender: TObject);
begin
  if GLevel = 0 then
    DoUnderSchoolGame
  else
  begin
    DoGameOne;
    ViewNumbers(IntToStr(CorrectCount), paCorrect, False);
    ViewNumbers(IntToStr(NotCorrectCount), paFalse, False);
  end;
  tiQuestion.enabled:= False;
  paState.Visible:= False;
end;

procedure TfmMain.tiTimeStartTimer(Sender: TObject);
begin

end;

procedure TfmMain.tiTimeTimer(Sender: TObject);
begin
  Remaining:= 300 - Trunc((Now - StartTime) * 24 * 60* 60);
  ViewNumbers(IntToStr(Remaining), paTime);
  if Remaining <= 0 then
   FinishGame

end;

function TfmMain.InvertNum(Num: string): string;
var
  i: Integer;
begin
  for i:= 1 to Length(Num) do
    Result:= Num[i] + Result;
end;

function TfmMain.InvertNumbers(Numbers: string): string;
var
  EON: Integer;
  ANum: string;
begin
  Result:= '';
  while Numbers <> '' do
  begin
    EON:= GetNumEnd(Numbers);
    ANum:= Copy(Numbers, 1, EON);
    ANum:= InvertNum(ANum);
    Delete(Numbers, 1, EON);
    Result:= Result + ANum;
    if Numbers <> '' then
    begin
      Result:= Result + Numbers[1];
      Delete(Numbers, 1, 1);
    end;
  end;

end;

function TfmMain.FindEmptyCell: Integer;
var
  i: Integer;
begin
  for i:= 0 to High(NumbersComp) do
    if NumbersComp[i] = nil then
    begin
      Result:= i;
      Break;
    end;
end;

procedure TfmMain.ViewNumbers(Numbers: string; var APanel: TPanel;
  Transparent: Boolean = False);
var
  i: Integer;
  Img: TImage;
  ImgName: string;
  SrcImg: TComponent;
  S: Integer;
begin
 { for i:= 0 to High(NumbersComp) do
    if (Assigned(NumbersComp[i])) and ((NumbersComp[i] as TImage).Parent = APanel) then
    begin
      FreeAndNil(NumbersComp[i]);
    end;}
  RemoveAllImagesInPanel(APanel);

  Numbers:= InvertNumbers(Numbers);
  S:= 0;
  for i:= 1 to Length(Numbers) do
  begin
    case Numbers[i] of
      '0' .. '9': ImgName:= 'im' + Numbers[i];
      '.': ImgName:= 'imPer';
      '+': ImgName:= 'imPls';
      '-': ImgName:= 'imMns';
      '*': ImgName:= 'imMlt';
      '/': ImgName:= 'imDiv';
      '=': ImgName:= 'imEql';
      '?': ImgName:= 'imQues';
    end;


    SrcImg:= FmNumberBox.FindComponent(ImgName);
    if SrcImg <> nil then
    begin
      Img:= TImage.Create(nil);
      Img.Picture.Assign(TImage(SrcImg).Picture);
      Img.Parent:= APanel;
      Img.Cursor:= APanel.Cursor;
      Img.OnClick:= APanel.OnClick;
      Img.Tag:= APanel.Tag;
      Img.Transparent:= Transparent;

      if not (Numbers[i] in ['0' .. '9', '.']) then
        Inc(S, 10);

      Img.Left:= APanel.Width - i * 20 - 50 - S;
      if (I <> Length(Numbers)) and (not (Numbers[i] in ['0' .. '9', '.'])) then
        Inc(S, 10);

      Img.Top:= 5;
      NumbersComp[FindEmptyCell]:= Img;
    end;

  end;

end;

procedure TfmMain.RemoveAllImagesInPanel(APanel: TPanel);
var
  i: Integer;
begin
  for i:= 0 to High(NumbersComp) do
    if (Assigned(NumbersComp[i])) and ((NumbersComp[i] as TImage).Parent = APanel) then
    begin
      FreeAndNil(NumbersComp[i]);
    end;

end;

function TfmMain.GetNumEnd(Numbers: string): Integer;
var
  i: Integer;
begin
  Result:= Length(Numbers);
  for i:= 1 to Length(Numbers) do
    if (not (Numbers[i] in ['0' .. '9', '.'])) then
    begin
      Result:= i - 1;
      Break;
    end;

end;

procedure TfmMain.DoGameOne;
var
  Formula: string;
  Answers: array [1 .. 6] of string;
  i: Integer;
  Numbers: string;
  EqPos: Integer;

  procedure SwapAnswers(Index1, Index2: Integer);
  var
    TempCor: Boolean;
    TempAns: string;
  begin
    TempAns:= Answers[Index1];
    TempCor:= Correct[Index1];
    Answers[Index1]:= Answers[Index2];
    Correct[Index1]:= Correct[Index2];
    Answers[Index2]:= TempAns;
    Correct[Index2]:= TempCor;
  end;

begin
  if Remaining <= 0 then
    Exit;
  Formula:= GetSimpleFormulaQuiz(GLevel);
  EqPos:= Pos('=', Formula);
  Answers[1]:= Copy(Formula, EqPos + 1, Length(Formula) - EqPos);
  Formula:= Copy(Formula, 1, Pos('=', Formula));
  Formula:= Formula + '?';
  ViewNumbers(Formula, paQuestion);
  Correct[1]:= True;
  Numbers:= Answers[1] + ',';
  for i:= 2 to 6 do
  begin
    repeat
      Formula:= GEtSimpleFormulaQuiz(Glevel);
      EqPos:= Pos('=', Formula);
      Answers[i]:= Copy(Formula, EqPos + 1, Length(Formula) - EqPos);
    until Pos(Answers[i] + ',', Numbers) = 0;
    Correct[i]:= False;
    Numbers:= Numbers + Answers[i] + ',';
  end;
  for i:= 1 to Random(100) + 50 do
  begin
    SwapAnswers(1, 3);

    SwapAnswers(2, 4);

    if GLevel > 4 then
      SwapAnswers(3, 5);

    if GLevel > 5 then
      SwapAnswers(6, 2);

    if Random(20) mod 2 = 0 then
      SwapAnswers(1, 2);

    if Random(20) mod 3 = 0 then
      SwapAnswers(2, 3);

  end;
  ViewNumbers(Answers[1], paAns1);
  ViewNumbers(Answers[2], paAns2);
  ViewNumbers(Answers[3], paAns3);
  ViewNumbers(Answers[4], paAns4);

  if GLevel > 4 then
    ViewNumbers(Answers[5], paAns5);

  if GLevel > 5 then
    ViewNumbers(Answers[6], paAns6);

  paAns1.Enabled:= True;
  paAns2.Enabled:= True;
  paAns3.Enabled:= True;
  paAns4.Enabled:= True;
  paAns5.Enabled:= True;
  paAns6.Enabled:= True;
end;

procedure TfmMain.DoUnderSchoolGame;
var
  x, y: Integer;
  Img: TImage;
  SrcImg: TComponent;
begin
  Randomize;
  x:= Random(9) + 1;
  y:= Random(5) + 1;
  UnderSNum:= x;

  SrcImg:= FmNumberBox.FindComponent('imn' + IntToStr(x) + '_' + IntToStr(y));
  if SrcImg <> nil then
  begin
    RemoveAllImagesInPanel(paQuestion);
    Img:= TImage.Create(nil);
    Img.Picture.Assign(TImage(SrcImg).Picture);
    Img.Parent:= paQuestion;
    Img.Transparent:= True;
    Img.Left:= 1;
    Img.Top:= 2;
    Img.Width:= 100;
    Img.Height:= 100;
    NumbersComp[FindEmptyCell]:= Img;
  end;

end;

procedure TfmMain.FinishGame;
var
  Mark: Integer;
begin
  tiTime.Enabled:= False;
  tiFinish.Enabled:= True;
  paAns1.Enabled:= False;
  paAns2.Enabled:= False;
  paAns3.Enabled:= False;
  paAns4.Enabled:= False;
  paAns5.Enabled:= False;
  paAns6.Enabled:= False;
  laLevelSelect.Enabled:= True;
  Mark:= GetMark;
  ViewNumbers(IntToStr(Mark), paMark);
  paMark.Visible:= True;
  SaveGame;
end;

procedure TfmMain.SaveGame;
var
  F: File of TPlayRec;
  Rec: TPlayRec;
  FileName: string;
  Mark: Integer;
begin
  try
    FileName:= ChangeFileExt(ParamStr(0), '.ply');
    AssignFile(F, FileName);
    if FileExists(FileName) then
    begin // Append
      FileMode:= 2;
      Reset(F);
      Seek(F, System.FileSize(F));
    end
    else // New file
      Rewrite(F);

    Mark:= GetMark;

    Rec.ATime:= StartTime;
    Rec.Correct:= CorrectCount;
    Rec.NotCorrect:= NotCorrectCount;
    Rec.Level:= GLevel;
    Rec.UserName:= UserName;
    Rec.Mark:= Mark;
    Write(F, Rec);
    CloseFile(F);

  except
  on e: exception do
  begin
    ShowMessage('خطأ أثناء حفظ النتيجة' + #10 + e.Message);
  end;
  end;

end;

function TfmMain.GetMark: Integer;
begin
  Result:= (CorrectCount * 2) - NotCorrectCount;
  if Result  < 0 then
    Result:= 0;
end;

initialization
  {$I main.lrs}

end.

